@@ -1,9 +1,7 @@
-#!/usr/bin/ruby -w
+#!/usr/bin/ruby
 
-$stderr.reopen('/dev/null', 'w')
 require "glib2"
 require "gtk2"
-$stderr=STDOUT
 
 require "eataria2"
 require "eatmanager"
