@@ -16,6 +16,14 @@ def debug(*args)
 	pp args
 end
 
+def debug_time(title, time_start, time_end)
+	$time_report = Hash.new if !defined? $time_report
+	$time_report[title] = Array.new if $time_report[title] == nil
+	$time_report[title] << time_end - time_start
+	$time_report[title].delete($time_report[title].first) if $time_report[title].length > 10
+	debug("time_report", title, "now", time_end - time_start, "#{$time_report[title].length} last sec", $time_report[title].inject(:+) / $time_report[title].length)
+end
+
 
 
 module Eat
@@ -49,6 +57,9 @@ class Eat::Aria2 < GLib::Object
 	signal_new("download_status", GLib::Signal::RUN_FIRST, nil,
 			nil,					# Return type: void
 			String)					# Parameters: gid
+	signal_new("download_waiting", GLib::Signal::RUN_FIRST, nil,
+			nil,					# Return type: void
+			String)					# Parameters: gid
 	signal_new("download_completed", GLib::Signal::RUN_FIRST, nil,
 			nil,					# Return type: void
 			String)					# Parameters: gid
@@ -58,25 +69,24 @@ class Eat::Aria2 < GLib::Object
 	signal_new("download_stopped", GLib::Signal::RUN_FIRST, nil,
 			nil,					# Return type: void
 			String)					# Parameters: gid
+	signal_new("download_error", GLib::Signal::RUN_FIRST, nil,
+			nil,					# Return type: void
+			String)					# Parameters: gid
 
-	def signal_do_connected()
-	end
-	def signal_do_disconnected()
-	end
-	def signal_do_shutdown()
-	end
-	def signal_do_download_status(gid)
-	end
-	def signal_do_download_completed(gid)
-	end
-	def signal_do_download_removed(gid)
-	end
-	def signal_do_download_stopped(gid)
-	end
+	def signal_do_connected(); end
+	def signal_do_disconnected(); end
+	def signal_do_shutdown(); end
+	def signal_do_download_status(gid); end
+	def signal_do_download_waiting(gid); end
+	def signal_do_download_completed(gid); end
+	def signal_do_download_removed(gid); end
+	def signal_do_download_stopped(gid); end
+	def signal_do_download_error(gid); end
 
 	def initialize(hostname = DEFAULT_HOST, port = DEFAULT_PORT, user = nil, password = nil)
 		super(nil)
 
+		# Store new downloads added through add_<FOO>
 		@new_downloads = []
 
 		@confdir = ENV['XDG_CONFIG_HOME']
@@ -88,6 +98,7 @@ class Eat::Aria2 < GLib::Object
 		@use_local_server = true
 		@version = "n/a"
 		@@client = nil if !defined? @@client
+		@connect_thread = nil
 		@shutdown_thread = nil
 
 		if !defined? @@pid
@@ -170,7 +181,7 @@ class Eat::Aria2 < GLib::Object
 =end
 	def spawn_server(command)
 		pid = Process.spawn(command, :pgroup=>true, :chdir=>ENV['HOME'],
-				STDOUT=>"/dev/null", STDIN=>"/dev/null")
+				STDOUT=>"/dev/null", STDERR=>"/dev/null", STDIN=>"/dev/null")
 		Process.detach(pid)
 		# Wait for the server to respond properly to requests
 		debug("waiting for a first response...")
@@ -192,6 +203,8 @@ class Eat::Aria2 < GLib::Object
 	Make an XML-RPC call to the server. Try to connect if the connection is broken.
 =end
 	def call(method, *args)
+		@connect_thread.join if @connect_thread != nil and Thread.current != @connect_thread
+		return if @@client == nil
 		begin
 			result = @@client.call(method, *args)
 		rescue XMLRPC::FaultException => e
@@ -229,7 +242,7 @@ class Eat::Aria2 < GLib::Object
 	def connect(force=false)
 		return if @@client and force == false
 		@is_connected = false
-		Thread.new do
+		@connect_thread = Thread.new do
 			@shutdown_thread.join if @shutdown_thread != nil
 			begin
 				debug("connect")
@@ -318,7 +331,7 @@ class Eat::Aria2 < GLib::Object
 
 	# Adds BitTorrent download by uploading .torrent file.
 	def add_torrent(torrent, uris = [], options = [], position = nil)
-		data = read(torrent)
+		data = File.read(torrent)
 		if position != nil
 			gid = call("aria2.addTorrent", XMLRPC::Base64.new(data), uris, options, position)
 		else
@@ -330,7 +343,7 @@ class Eat::Aria2 < GLib::Object
 
 	# Adds Metalink download by uploading .metalink file.
 	def add_metalink(metalink, options = [], position = nil)
-		data = read(metalink)
+		data = File.read(metalink)
 		if position != nil
 			gid = call("aria2.addMetalink", XMLRPC::Base64.new(data), options, position)
 		else
@@ -427,13 +440,20 @@ end
 
 
 
+=begin
+Eat::Aria2Listener:
+Polls each second the state of active downloads and caches the status for each
+active download inside the public (read-only) property @downloads.
+=end
 class Eat::Aria2Listener < Eat::Aria2
 
 	include Singleton
 
+	attr_reader :downloads
+
 	def initialize()
 		super
-		@downloads = []
+		@downloads = {}
 		start
 	end
 
@@ -445,40 +465,33 @@ class Eat::Aria2Listener < Eat::Aria2
 			if @is_connected
 
 				# Emit status signal for each active download
-				tell_active.each do |res|
-					gid = res["gid"]
-					@downloads << gid if !@downloads.find_index(gid)
+#		time_start = Time.now
+				tell_active.each do |status|
+					gid = status["gid"]
 					active_downloads << gid
+					@downloads[gid] = status
 					signal_emit("download_status", gid)
 				end
-
-				# Check/Cleanup unhandled new downloads
-				(@new_downloads - active_downloads).each do |gid|
-					@downloads << gid
-				end
-				@new_downloads.clear
+#		time_end = Time.now
+#		debug_time("active downloads", time_start, time_end)
 
 				# Emit specific signal for each inactive download
-				(@downloads - active_downloads).each do |gid|
-					# Get status
+				(@new_downloads - active_downloads).each do |gid|
+					# Store status
 					status = tell_status(gid)
+					@downloads[gid] = status
 					case status["status"]
 						when "complete" then signal_emit("download_completed", gid)
+						when "waiting" then signal_emit("download_waiting", gid)
 						when "removed" then signal_emit("download_removed", gid)
-						when "error"
-							if status["errorCode"] == "5"
-								# Download was aborted due to "pause"
-								debug "pause lasted too long, download %s dropped" % gid
-							else
-								puts "Unhandled error (%s: %s)" % [gid, status["errorCode"]]
-							end
-						else puts "Unhandled status (%s: %s)" % [gid, status["status"]]
+						when "error" then signal_emit("download_error", gid)
+						else puts "Inconsistent status (%s: %s)" % [gid, status["status"]]
 					end
+					@new_downloads.delete(gid)
 				end
 
 			end
 
-			@downloads = active_downloads
 			true
 		end
 	end
@@ -677,25 +690,23 @@ end
 
 
 if __FILE__ == $0
-	aria2 = Eat::Aria2.new
-	exit if !aria2.is_connected
-	puts "Version: aria2 "+aria2.version
-	#aria2.add_uri(["http://dlc.sun.com/torrents/info/osol-0906-x86.iso.torrent"])
-	begin
-		aria2.tell_active.each do |res|
-			gid = res["gid"]
-			puts " *** Set download max rate to 16KB ***"
-			aria2.change_options(gid, "max-download-limit" => "16000")
-			puts " *** Get status ***"
-			pp aria2.tell_status(gid)
-			puts " *** Get uris ***"
-			pp aria2.get_uris(gid)
-			puts " *** Get files ***"
-			pp aria2.get_files(gid)
-			puts " *** Get peers ***"
-			pp aria2.get_peers(gid).length
-		end
-		sleep 5
-	end while true
+	aria2 = Eat::Aria2Listener.instance
+	aria2.connect
+	Thread.new do
+		begin
+			if aria2.is_connected
+				puts ""
+				puts " ****** DOWNLOADS STATUS ****** "
+				puts ""
+				aria2.downloads.each do |gid, status|
+					puts "#{gid}: ", status
+					puts " *** Get peers ***"
+					pp aria2.get_peers(gid).length
+				end
+			end
+		end while sleep 5
+	end
+	mainloop = GLib::MainLoop.new
+	mainloop.run
 end
 
