@@ -22,26 +22,6 @@ class Eat::Manager
 		# Store manually removed downloads to avoid readding them into the tree store
 		@removed_downloads = Array.new
 
-		# Setup aria2 listener
-		@aria2 = Eat::Aria2Listener.instance
-		@aria2.signal_connect("connected") {
-			@removed_downloads.clear
-			update_newdl_dialog
-			set_sensitive(true)
-			@infobar.hide_all
-			@infobar.set_no_show_all(true)
-		}
-		@aria2.signal_connect("disconnected") {
-			set_sensitive(false)
-			if !@aria2.use_local_server?
-				@infobar.set_no_show_all(false)
-				@infobar.show_all
-			end
-		}
-		@aria2.signal_connect("download_status") { |this, gid| update_row(gid) }
-		@aria2.signal_connect("download_completed") { |this, gid| update_row(gid) }
-		@aria2.signal_connect("download_removed") { |this, gid| update_row(gid) }
-
 		# Retrieve GUI widgets
 		builder = Builder.new
 		builder << "manager.ui"
@@ -68,6 +48,25 @@ class Eat::Manager
 		@action_remove = builder["action-remove"]
 		@action_settings = builder["action-settings"]
 
+		# Setup aria2 listener
+		@aria2 = Eat::Aria2Listener.instance
+		@aria2.signal_connect("connected") {
+			@removed_downloads.clear
+			update_newdl_dialog
+			set_sensitive(true)
+			@infobar.hide
+		}
+		@aria2.signal_connect("disconnected") {
+			set_sensitive(false)
+			if !@aria2.use_local_server?
+				@infobar.show_error("Unable to connect to aria2", "Reconnect", lambda { @aria2.connect(true) })
+			end
+		}
+		@aria2.signal_connect("download_status") { |this, gid| update_row(gid) }
+		@aria2.signal_connect("download_completed") { |this, gid| update_row(gid) }
+		@aria2.signal_connect("download_removed") { |this, gid| update_row(gid) }
+		@aria2.signal_connect("download_error") { |this, gid| update_row(gid) }
+
 		# Setup actions
 		@action_add.signal_connect("activate") { action_add }
 		@action_pause.signal_connect("activate") { action_pause }
@@ -81,22 +80,8 @@ class Eat::Manager
 		@window.signal_connect('delete-event') { action_quit }
 
 		# Setup info bar
-=begin
 		# ruby-gnome2-0.19.3 doesn't know about Gtk::InfoBar
-		# The info bar will be used at least in one case where the connection gets
-		# refused. It will provide a visible message with an action button to manually
-		# try to reconnect.
-		@infobar = InfoBar.new
-=end
-		@infobar = HBox.new(false, 4)
-		@infobar.set_no_show_all(true)
-		infobutton = Button.new("Reconnect")
-		infobutton.signal_connect('clicked') { @aria2.connect(true) }
-		@infobar.pack_end(infobutton, false, false, 0)
-		infolabel = Label.new("Disconnected from aria2")
-		@infobar.pack_end(infolabel, false, false, 0)
-		infoimg = Image.new(Stock::DIALOG_ERROR, IconSize::MENU)
-		@infobar.pack_end(infoimg, false, false, 0)
+		@infobar = Eat::ManagerInfoBar.new
 		hbox = builder["hbox-info-bar"]
 		hbox.add(@infobar)
 
@@ -143,12 +128,30 @@ class Eat::Manager
 		file_filter.add_pattern("*.torrent")
 		file_filter.add_pattern("*.metalink")
 		@file_select_file_dialog.filter = file_filter
-		@file_select_file.signal_connect('clicked') do
+		@file_select_file.signal_connect('clicked') {
 			@file_select_file_dialog.unselect_all
 			res = @file_select_file_dialog.run
 			@file_uri.text = @file_select_file_dialog.filename if res == Gtk::Dialog::RESPONSE_ACCEPT
 			@file_select_file_dialog.hide
-		end
+		}
+
+		# Setup notification icon
+		@statusicon = StatusIcon.new
+		@statusicon.set_icon_name("eatmonkey")
+		@statusicon.signal_connect("activate") {
+			# Check if window is invisible
+			if not @window.flags & Widget::VISIBLE
+				@window.show
+			# Check if window is inactive
+			elsif not @window.active?
+				@window.present
+			# Else hide
+			else
+				winx, winy = @window.position
+				@window.hide
+				@window.move(winx, winy)
+			end
+		}
 	end
 
 	def show()
@@ -173,8 +176,8 @@ class Eat::Manager
 		return if @removed_downloads.include? gid_i
 
 		# Retrieve status information on gid
-		status = @aria2.tell_status(gid)
-		return if status.empty?
+		status = @aria2.downloads[gid]
+		return if status == nil
 
 		# Find gid in model
 		@liststore.each do |model, path, iter|
@@ -225,7 +228,7 @@ class Eat::Manager
 		when "removed" then
 			# TODO mark as stopped/inactive
 		when "error" then
-			row_iter[8] = -1
+			@infobar.show_error("Download error (%s) on %s" % [status["errorCode"], row_iter[9]], "Close")
 		end
 	end
 
@@ -306,16 +309,21 @@ class Eat::Manager
 	end
 
 	def add_new_download(uri=nil)
+		# Update dialog
 		update_newdl_dialog
 		@file_uri.text = uri == nil ? "" : uri
 		@file_uri.grab_focus
+
+		# Run dialog
 		res = @newdl_dialog.run
 		@newdl_dialog.hide
+
+		# Read URI
 		uri = @file_uri.text
+
 		if res == Dialog::RESPONSE_OK and !uri.empty?
-			# TODO check if it is a uri or a torrent/metalink file and use the right
-			# method addUri/addTorrent/addMetalink
-			puts "download file %s" % uri
+			# Prepare options
+			puts "download uri: %s" % uri
 			options = {
 				"max-download-limit" => @file_max_download_speed.value_as_int.to_s,
 				"split" => @file_split.value_as_int.to_s,
@@ -323,14 +331,30 @@ class Eat::Manager
 				"seed-ratio" => @file_seed_ratio.value.to_s,
 			}
 			options["dir"] = @file_download_dir.current_folder if @aria2.use_local_server?
-			gid = @aria2.add_uri(uri, options)
+
+			# Add download to aria2
+			gid = nil
+			if File.readable? uri
+				if uri.end_with? "torrent"
+					gid = @aria2.add_torrent(uri, options)
+				elsif uri.end_with? "metalink"
+					gid = @aria2.add_metalink(uri, options)
+				end
+			else
+				gid = @aria2.add_uri(uri, options)
+			end
+			puts "gid: %s" % gid
+
+			# Update GUI
 			if gid != nil
-				puts "gid: %s" % gid
 				# Add new row to liststore
 				row_iter = @liststore.append
 				row_iter[0] = gid.to_i
 				row_iter[9] = File.basename(uri)
 				row_iter[11] = uri
+			else
+				# Display an error message
+				@infobar.show_error("The URI couldn't be added", "Close")
 			end
 		end
 	end
@@ -402,7 +426,7 @@ class Eat::Manager
 		# Resume gid that have been removed
 		@treeview.selection.selected_each do |model, path, iter|
 			gid = iter[0].to_s
-			status = @aria2.tell_status(gid)
+			status = @aria2.downloads[gid]
 			case status["status"]
 			when "removed"
 				# Restart the download queued at position 0 and delete current row
@@ -411,7 +435,7 @@ class Eat::Manager
 				if !uri.empty?
 					percent = iter[8]
 					@liststore.remove(iter)
-					gid = @aria2.add_uri(uri, nil, 0)
+					gid = @aria2.add_uri(uri, [], 0)
 					if gid != nil
 						# TODO this is a little better than nothing, when
 						# the download is resumed it will reappear immediately
@@ -440,7 +464,60 @@ end
 
 
 
+class Eat::ManagerInfoBar < HBox
+
+	private
+
+	def initialize()
+		super
+		set_homogeneous(false)
+		set_spacing(4)
+		set_no_show_all(true)
+		set_border_width(2)
+
+		@callback_id = 0
+		@button = Button.new
+		@button.signal_connect_after('clicked') do
+			hide
+		end
+		pack_end(@button, false, false, 0)
+
+		img = Image.new(Stock::DIALOG_ERROR, IconSize::MENU)
+		pack_start(img, false, false, 0)
+
+		@label = Label.new
+		@label.set_ellipsize(Pango::Layout::ELLIPSIZE_END)
+		@label.set_xalign(0.0)
+		@label.set_use_markup(true)
+		pack_start(@label, true, true, 0)
+
+		signal_connect("hide") {
+			if @button.signal_handler_is_connected?(@callback_id)
+				@button.signal_handler_disconnect(@callback_id)
+			end
+			set_no_show_all(true)
+		}
+	end
+
+	public
+
+	def show_error(message, button_label, button_callback = nil)
+		set_no_show_all(false)
+		show_all
+
+		@label.label = message
+		@button.label = button_label
+
+		@callback_id = @button.signal_connect('clicked') { button_callback.call } if button_callback != nil
+	end
+
+end
+
+
+
 if __FILE__ == $0
+	aria2 = Eat::Aria2Listener.instance
+	aria2.connect
 	manager = Eat::Manager.new
 	manager.show
 	main
