@@ -157,6 +157,7 @@ class Eat::SettingsDialog
 		@advanced[:enable_upnp] = builder["enable-upnp"]
 		@advanced[:certificates] = builder["certificates"]
 		@advanced[:save_ondisk] = builder["save-ondisk"]
+		@advanced[:prealloc] = builder["pre-alloc"]
 
 		# Setup parent window
 		@dialog.set_transient_for(parent_window)
@@ -186,6 +187,7 @@ class Eat::SettingsDialog
 		@advanced[:enable_upnp].active = @settings["enable-upnp"]
 		@advanced[:certificates].active = @settings.aria2["check-certificate"]
 		@advanced[:save_ondisk].active = @settings.aria2["follow-torrent"] == true
+		@advanced[:prealloc].active = @settings.aria2["file-allocation"] == "prealloc"
 
 		# Setup advanced configuration treeview
 		@treestore = TreeStore.new(String, String, TrueClass)
@@ -259,6 +261,7 @@ class Eat::SettingsDialog
 			@treeiter["listen-port"].set_value(1, @advanced[:listen_port].value_as_int.to_s)
 			@treeiter["check-certificate"].set_value(1, @advanced[:certificates].active?.to_s)
 			@treeiter["follow-torrent"].set_value(1, @advanced[:save_ondisk].active? ? "true" : "mem")
+			@treeiter["file-allocation"].set_value(1, @advanced[:prealloc].active? ? "prealloc" : "none")
 		else
 			# Sync widget values with treestore
 			@basic_options[:max_concurrent_downloads].value = @treeiter["max-concurrent-downloads"].get_value(1).to_i
@@ -275,6 +278,7 @@ class Eat::SettingsDialog
 			@advanced[:listen_port].value = @treeiter["listen-port"].get_value(1).to_i
 			@advanced[:certificates].active = @treeiter["check-certificate"].get_value(1) == "true"
 			@advanced[:save_ondisk].active = @treeiter["follow-torrent"].get_value(1) == "true"
+			@advanced[:prealloc].active = @treeiter["file-allocation"].get_value(1) == "prealloc"
 		end
 	end
 
@@ -306,6 +310,7 @@ class Eat::SettingsDialog
 			@settings.aria2["check-certificate"] = @advanced[:certificates].active?
 			@settings.aria2["follow-torrent"] = @advanced[:save_ondisk].active? ? "true" : "mem"
 			@settings.aria2["follow-metalink"] = @advanced[:save_ondisk].active? ? "true" : "mem"
+			@settings.aria2["file-allocation"] = @advanced[:prealloc].active? ? "prealloc" : "none"
 		end
 
 		@settings.save
